--------------------------------------------------
-- Iir8b12b.vhd
-- by Toshio Iwata at digitalfilter.com 2007/07/18

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_arith.all;

entity Iir8b12b is
   Port (
          MCLK : In std_logic;
          XIN : In std_logic_vector(7 downto 0);
          YOUT : Out std_logic_vector(7 downto 0) );
end Iir8b12b;

architecture RTL of Iir8b12b is
  signal X0, X1, X2 : signed(7 downto 0);
  signal Y1, Y2 : signed(7 downto 0);
  signal A0, A1, A2 : signed(11 downto 0);
  signal B1, B2 : signed(11 downto 0);
  signal HnXn : signed(19 downto 0);
  signal HnXn_16b : signed(15 downto 0);
  signal XIN_sig : signed(7 downto 0);
  signal YOUT_sig : signed(17 downto 0);
  signal YOUT_round : signed(17 downto 0);
  signal ADDOUT : signed(17 downto 0);
  signal co10 : unsigned(3 downto 0);
  signal MuxXn : signed(7 downto 0);
  signal MuxHn : signed(11 downto 0);

begin -- Iir8b12b
-------------------------------------------------------------------
gen_co10 : process( MCLK )
begin
  if( MCLK'event and MCLK = '1' ) then
    if( co10 = 9 ) then 
      co10 <= (others => '0');
    else 
      co10 <= co10 + 1;
    end if;
  end if;
end process;

-------------------------------------------------------------------
gen_MuxXn : process( MCLK )
begin
  if( MCLK'event and MCLK = '1' ) then
    if( co10 = 0 ) then 
      MuxXn <= X0;
    elsif( co10 = 1 ) then 
      MuxXn <= X1;
    elsif( co10 = 2 ) then 
      MuxXn <= X2;
    elsif( co10 = 3 ) then 
      MuxXn <= Y1;
    elsif( co10 = 4 ) then 
      MuxXn <= Y2;
    else 
      MuxXn <= (others => '0');
    end if;
  end if;
end process;

-------------------------------------------------------------------
gen_MuxHn : process( MCLK )
begin
  if( MCLK'event and MCLK = '1' ) then
    if( co10 = 0 ) then 
      MuxHn <= A0;
    elsif( co10 = 1 ) then 
      MuxHn <= A1;
    elsif( co10 = 2 ) then 
      MuxHn <= A2;
    elsif( co10 = 3 ) then 
      MuxHn <= B1;
    elsif( co10 = 4 ) then 
      MuxHn <= B2;
    else 
      MuxHn <= (others => '0');
    end if;
  end if;
end process;

-------------------------------------------------------------------
  HnXn <= MuxXn * MuxHn;

-------------------------------------------------------------------
gen_XIN_sig : process( XIN )
begin
  for i in 7 downto 0 loop
    XIN_sig(i) <= XIN(i);
  end loop;
end process;

----------------------------------------------------
gen_Xn : process( MCLK )
begin 
  if( MCLK'event and MCLK = '1' ) then
    if( co10 = 9 ) then 
      X2 <= X1;
      X1 <= X0;
      X0 <= XIN_sig;
    end if;
  end if;
end process;

----------------------------------------------------
gen_HnXn_16b: process( MCLK )
begin
  if( MCLK'event and MCLK = '1' ) then
    for i in 15 downto 0 loop
      HnXn_16b(i) <= HnXn(i+3);
    end loop;
  end if;
end process;

----------------------------------------------------
  ADDOUT <= HnXn_16b + YOUT_sig;

----------------------------------------------------
gen_YOUT_sig : process( MCLK )
variable tmpint : signed(21 downto 0);
begin
  if( MCLK'event and MCLK = '1' ) then
    if( co10 = 9 ) then 
      YOUT_sig <= (others => '0');
    else
      YOUT_sig <= ADDOUT;
    end if;
  end if;
end process;

----------------------------------------------------
  YOUT_round <= YOUT_sig + 64; -- to round it in bit 7

-------------------------------------------------------------------
gen_Y1 : process( MCLK )
begin
  if( MCLK'event and MCLK = '1' ) then
    if( co10 = 9 ) then 
      if( YOUT_round(17) = '0' and -- positive value
         ( YOUT_round(16) = '1' 
        or YOUT_round(15) = '1' 
        or YOUT_round(14) = '1' ) ) then -- overflow
        Y1 <= "01111111";
      elsif( YOUT_round(17) = '1' and -- negative value
         ( YOUT_round(16) = '0' 
        or YOUT_round(15) = '0' 
        or YOUT_round(14) = '0' ) ) then -- underflow
        Y1 <= "10000000";
      else 
        for i in 7 downto 0 loop
          Y1(i) <= YOUT_round(i+7);
        end loop;
      end if;
    end if;
  end if;
end process;

----------------------------------------------------
gen_Yn : process( MCLK )
begin 
  if( MCLK'event and MCLK = '1' ) then
    if( co10 = 9 ) then 
      Y2 <= Y1;
    end if;
  end if;
end process;

----------------------------------------------------
gen_YOUT : process( Y1 )
begin
  for i in 7 downto 0 loop
    YOUT(i) <= Y1(i);
  end loop;
end process;

----------------------------------------------------
----------------------------------------------------
--U_0
--Chebychev LPF
--Sampling Frequency = 5000000.0
--cutoff1 =  100000.0
--Pass Band Ripple =0.300000
--Order = 2nd
--Quantized by 11 [bits]
--**** 1st buquad ****
--a0 = 000000000110 
--a1 = 000000001101 
--a2 = 000000000110 
--b1 = 100011100001 
--b2 = 001100111100 
  A0 <= "000000000110";
  A1 <= "000000001101";
  A2 <= "000000000110";
  B1 <= "011100011111"; -- polarity is inverted
  B2 <= "110011000100"; -- polarity is inverted

--U_3
--Chebychev HPF
--Sampling Frequency = 5000000.0
--cutoff1 =  250000.0
--Pass Band Ripple =0.300000
--Order = 2nd
--Quantized by 11 [bits]
--**** 1st buquad ****
--a0 = 001101011010 
--a1 = 100101001011 
--a2 = 001101011010 
--b1 = 100100100101 
--b2 = 001100001001 
--  A0 <= "001101011010";
--  A1 <= "100101001011";
--  A2 <= "001101011010";
--  B1 <= "011011011011"; -- polarity is inverted
--  B2 <= "110011110111"; -- polarity is inverted

--  A0 <= "001111111111";
--  A1 <= "000000000000";
--  A2 <= "000000000000";
--  B1 <= "000000000000"; -- polarity is inverted
--  B2 <= "000000000000"; -- polarity is inverted

end; -- Iir8b12b
